/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __ARCHIVE_H__
#define __ARCHIVE_H__

#include <limits.h>

struct s_dico;

struct s_archive
{
	int	archfd; // file descriptor of the current volume (set to -1 when closed)
	u32	archid; // 32bit archive id for checking (random number generated at creation)
	u64	fscount; // how many filesystems in archive (valid only if archtype=filesystems)
	int 	locked; // true if the file is locked by the open/create/close functions
	u32	archtype; // what has been saved in the archive: filesystems or directories
	u32	curvol; // current volume number, starts at 0, incremented when we change the volume
	char	filefmt[FSA_MAX_FILEFMTLEN]; // file format of that archive
	char 	creatver[FSA_MAX_PROGVERLEN]; // fsa version used to create archive
	char	label[FSA_MAX_LABELLEN]; // archive label defined by the user
	char	basepath[PATH_MAX]; // path of the first volume of an archive
	char	volpath[PATH_MAX]; // path of the current volume of an archive
};

int archive_init(struct s_archive *ai);
int archive_destroy(struct s_archive *ai);
int archive_create(struct s_archive *ai);
int archive_open(struct s_archive *ai);
int archive_close(struct s_archive *ai);
int archive_remove(struct s_archive *ai);
int archive_generate_id(struct s_archive *ai);
s64 archive_get_currentpos(struct s_archive *ai);

int archive_read_data(struct s_archive *ai, void *data, u64 size);
int archive_read_dico(struct s_archive *ai, struct s_dico *d);
int archive_read_header(struct s_archive *ai, char *magic, struct s_dico **d, bool allowseek);
int archive_write_data(struct s_archive *ai, void *data, u64 size);
int archive_incvolume(struct s_archive *ai, bool waitkeypress);
int archive_volpath(struct s_archive *ai);

#endif // __ARCHIVE_H__
