/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __DICO_H
#define __DICO_H

#include "types.h"

#define DICO_MAX_DATASIZE	16384

enum	{DICO_ESUCCESS=0, DICO_ENOENT, DICO_EINVAL, DICO_EBADSIZE, DICO_EFULL, DICO_EMEM, DICO_EDUPLICATE, DICO_EINVALCHAR};
enum	{DICTYPE_NULL=0, DICTYPE_DATA, DICTYPE_STRING, DICTYPE_U8, DICTYPE_U16, 
	DICTYPE_U32, DICTYPE_U64, DICTYPE_S8, DICTYPE_S16, DICTYPE_S32, DICTYPE_S64};

struct s_dico { struct s_item *head; };

struct s_item
{
	u8		section;
	u64		key;
	char		*data;
	u16		size;
	int		type;
	struct s_item	*next;
};

struct s_dico *dico_alloc(); // dynamic memory allocation
int dico_destroy(struct s_dico *d);
int dico_show(struct s_dico *d, u8 section, char *debugtxt);
int dico_count_all_sections(struct s_dico *d);
int dico_count_one_section(struct s_dico *d, u8 section);
int dico_add_data(struct s_dico *d, u8 section, u64 key, const void *data, u16 size);
int dico_add_internal(struct s_dico *d, u8 section, u64 key, const void *data, u16 size, int type);
int dico_get_internal(struct s_dico *d, u8 section, u64 key, void *data, u16 maxsize, u16 *size);
int dico_get_data(struct s_dico *d, u8 section, u64 key, void *data, u16 maxsize, u16 *size);
int dico_add_u32(struct s_dico *d, u8 section, u64 key, u32 data);
int dico_add_s32(struct s_dico *d, u8 section, u64 key, s32 data);
int dico_add_u64(struct s_dico *d, u8 section, u64 key, u64 data);
int dico_add_s64(struct s_dico *d, u8 section, u64 key, s64 data);
int dico_get_u32(struct s_dico *d, u8 section, u64 key, u32 *data);
int dico_get_s32(struct s_dico *d, u8 section, u64 key, s32 *data);
int dico_get_u64(struct s_dico *d, u8 section, u64 key, u64 *data);
int dico_get_s64(struct s_dico *d, u8 section, u64 key, s64 *data);
int dico_add_string(struct s_dico *d, u8 section, u64 key, const char *szstring);
int dico_get_string(struct s_dico *d, u8 section, u64 key, char *buffer, u16 bufsize);

#endif // __DICO_H
