/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __FSARCHIVER_H__
#define __FSARCHIVER_H__

#include "config.h"
#include "types.h"
#include "error.h"

// global variables
extern struct s_options g_options;
extern char *valid_magic[];
extern int g_logfile;

#if !defined(min)
#	define min(a, b)       ((a) < (b) ? (a) : (b))
#endif

#if !defined(max)
#	define max(a, b)       ((a) > (b) ? (a) : (b))
#endif

// ----------------------------- fsarchiver const ---------------------------------------------------
#define FSA_VERSION		PACKAGE_VERSION
#define FSA_RELDATE		PACKAGE_RELDATE
#define FSA_FILEFORMAT		PACKAGE_FILEFMT

#define FSA_MAX_FILEFMTLEN	32
#define FSA_MAX_PROGVERLEN	32
#define FSA_MAX_LABELLEN	512
#define FSA_MAX_FSNAMELEN	128

#define FSA_MAX_HEADSIZE	65536
#define FSA_MAX_FSPERARCH	128
#define FSA_MAX_COMPJOBS	32
#define FSA_MAX_QUEUESIZE	32
#define FSA_MAX_BLKSIZE		921600
#define FSA_DEF_BLKSIZE		262144

// ----------------------------- fsarchiver magics --------------------------------------------------
#define FSA_SIZEOF_MAGIC	4
#define FSA_MAGIC_VOLH		"FsA0" // volume header (one per volume at the very beginning)
#define FSA_MAGIC_VOLF		"FsAE" // volume footer (one per volume at the very end)
#define FSA_MAGIC_MAIN		"ArCh" // archive header (one per archive at the beginning of the first volume)
#define FSA_MAGIC_FSIN		"FsIn" // filesys info (one per filesystem at the beginning of the archive)
#define FSA_MAGIC_FSYB		"FsYs" // filesys begin (one per filesystem when the filesys contents start, or when the flatfiles start)
#define FSA_MAGIC_OBJT		"ObJt" // object header (one per object: regfiles, dirs, symlinks, ...)
#define FSA_MAGIC_BLKH		"BlKh" // datablk header (one per data block, each regfile may have [0-n])
#define FSA_MAGIC_FILF		"FiLf" // filedat footer (one per regfile, after the list of data blocks)
#define FSA_MAGIC_DATF		"DaEn" // data footer (one per file system, at the end of its contents, or after the contents of the flatfiles)

// ------------ struct that stores the options passed on the command line ---------------------------
struct s_options
{
	u8	overwrite;
	u8	allowsaverw;
	int	verboselevel;
	int	debuglevel;
	int	compresslevel;
	int	compressjobs;
	u64	splitsize;
};

// -------------------------------- fsarchiver commands ---------------------------------------------
enum	{OPER_NULL=0, OPER_SAVEFS, OPER_RESTFS, OPER_SAVEDIR, OPER_RESTDIR, OPER_ARCHINFO};

// ----------------------------------- dico sections ------------------------------------------------
enum	{DICO_OBJ_SECTION_STDATTR=0, DICO_OBJ_SECTION_XATTR=1};

// ----------------------------------- archive types ------------------------------------------------
enum	{ARCHTYPE_NULL=0, ARCHTYPE_FILESYSTEMS, ARCHTYPE_DIRECTORIES};

// ----------------------------------- volume header and footer -------------------------------------
enum	{VOLUMEHEADKEY_VOLNUM, VOLUMEHEADKEY_ARCHID, VOLUMEHEADKEY_FILEFORMATVER, VOLUMEHEADKEY_PROGVERCREAT};
enum	{VOLUMEFOOTKEY_VOLNUM, VOLUMEFOOTKEY_ARCHID, VOLUMEFOOTKEY_LASTVOL};

// ----------------------------------- dico keys ----------------------------------------------------
enum	{OBJTYPE_NULL=0, OBJTYPE_DIR, OBJTYPE_SYMLINK, OBJTYPE_REGFILE, OBJTYPE_HARDLINK, 
	OBJTYPE_CHARDEV, OBJTYPE_BLOCKDEV, OBJTYPE_FIFO, OBJTYPE_SOCKET};
enum	{COMPRESS_NULL=0, COMPRESS_NONE, COMPRESS_LZO, COMPRESS_GZIP, COMPRESS_BZIP2, COMPRESS_LZMA};
enum	{DISKITEMKEY_NULL=0, DISKITEMKEY_FSID, DISKITEMKEY_PATH, DISKITEMKEY_OBJTYPE, DISKITEMKEY_SYMLINK, 
	DISKITEMKEY_HARDLINK, DISKITEMKEY_RDEV, DISKITEMKEY_MODE, DISKITEMKEY_SIZE, DISKITEMKEY_UID, 
	DISKITEMKEY_GID, DISKITEMKEY_ATIME, DISKITEMKEY_MTIME};
enum	{BLOCKHEADITEMKEY_NULL=0, BLOCKHEADITEMKEY_REALSIZE, BLOCKHEADITEMKEY_BLOCKOFFSET, 
	BLOCKHEADITEMKEY_COMPRESSALGO, BLOCKHEADITEMKEY_ARSIZE, BLOCKHEADITEMKEY_ARCSUM};
enum	{BLOCKFOOTITEMKEY_NULL=0, BLOCKFOOTITEMKEY_MD5SUM};
enum	{MAINHEADKEY_NULL=0, MAINHEADKEY_FILEFORMATVER, MAINHEADKEY_PROGVERCREAT, MAINHEADKEY_ARCHIVEID, 
	MAINHEADKEY_CREATTIME, MAINHEADKEY_ARCHLABEL, MAINHEADKEY_ARCHTYPE};
enum	{FSYSHEADKEY_NULL=0, FSYSHEADKEY_FILESYSTEM, FSYSHEADKEY_MNTPATH, FSYSHEADKEY_BYTESTOTAL, 
	FSYSHEADKEY_BYTESUSED, FSYSHEADKEY_FSLABEL, FSYSHEADKEY_FSUUID, FSYSHEADKEY_FSINODESIZE, 
	FSYSHEADKEY_FSVERSION, FSYSHEADKEY_FSEXTDEFMNTOPT, MAINHEADKEY_FSCOUNT, FSYSHEADKEY_FSEXTFEATURES, 
	FSYSHEADKEY_FSEXTBLOCKSIZE, FSYSHEADKEY_FSREISERBLOCKSIZE, FSYSHEADKEY_FSREISER4BLOCKSIZE, 
	FSYSHEADKEY_FSXFSBLOCKSIZE, FSYSHEADKEY_FSBTRFSSECTORSIZE};
enum	{FSINFOKEY_NULL=0, FSINFOKEY_FSID, FSINFOKEY_DEST, FSINFOKEY_MKFS};

#endif // __FSARCHIVER_H__
