/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#include "fsarchiver.h"

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <endian.h>
#include <byteswap.h>
#include <signal.h>
#include <linux/limits.h>

#include "fsarchiver.h"
#include "syncthread.h"

// queue use to share data between the three sort of threads
struct s_queue g_queue;

// g_stopfillqueue is set to true when the threads that reads the queue wants to stop
// either because there is an error or because it does not need the next data
atomic_t g_stopfillqueue={ (false) };

void set_stopfillqueue()
{	
	atomic_set(&g_stopfillqueue, true);
}

bool get_stopfillqueue()
{
	return atomic_read(&g_stopfillqueue);
}

// how many secondary threads are running (compression/decompression and archio threads)
atomic_t g_secthreads={ (0) };

void inc_secthreads()
{
	(void)__sync_add_and_fetch(&g_secthreads.counter, 1);
}

void dec_secthreads()
{
	(void)__sync_sub_and_fetch(&g_secthreads.counter, 1);
}

int get_secthreads()
{
	return atomic_read(&g_secthreads);
}

bool get_interrupted()
{
	return (get_abort()==true) || (get_stopfillqueue()==true);
}

// get_abort() returns true if a SIGINT/SIGTERM has been received (interrupted by the user)
int get_abort()
{
	int mysigs[]={SIGINT, SIGTERM, -1};
	sigset_t mask_set;
	sigpending(&mask_set);
	int i;
	
	if (sigpending(&mask_set)==0)
	{	
		for (i=0; mysigs[i]!=-1; i++)
		{
			if (sigismember(&mask_set, mysigs[i]))
			{	msgprintf(MSG_FORCE, "get_terminate(): received signal %d\n", mysigs[i]);
				return true;
			}
		}
	}
	
	return false;
}
