#include <stdio.h>
#include <lzma.h>
#include <string.h>

#define BUFLEN	262144

/*typedef enum {
	LZMA_OK                 = 0,
	LZMA_STREAM_END         = 1,
	LZMA_NO_CHECK           = 2,
	LZMA_UNSUPPORTED_CHECK  = 3,
	LZMA_GET_CHECK          = 4,
	LZMA_MEM_ERROR          = 5,
	LZMA_MEMLIMIT_ERROR     = 6,
	LZMA_FORMAT_ERROR       = 7,
	LZMA_OPTIONS_ERROR      = 8,
	LZMA_DATA_ERROR         = 9,
	LZMA_BUF_ERROR          = 10,
	LZMA_PROG_ERROR         = 11,
};*/

int main()
{
	char bufferin[BUFLEN];
	char bufferout[BUFLEN];
	lzma_stream lzma = LZMA_STREAM_INIT;
	FILE *fin, *fout;
	long origsize;
	long finalsize;
	int res;
	
	// init
	memset(bufferin, 0, sizeof(bufferin));
	memset(bufferout, 0, sizeof(bufferout));
	
	// ---- 1. COMPRESSION
	fin=fopen("/root/testlzma/file1.txt", "rb");
	if (!fin)
	{	printf ("Cannot open file1.txt\n");
		lzma_end(&lzma);
		return 1;
	}
	origsize=fread(bufferin, 1, BUFLEN, fin);
	printf ("fread(file1)=%d\n", origsize);
	if (!feof(fin))
	{	fclose(fin);
		printf("file is too big for the test. Must be smaller than 262144 bytes\n");
		lzma_end(&lzma);
		return 1;
	}
	fclose(fin);
	
	// init lzma structures
	lzma.next_in = bufferin;
	lzma.avail_in = origsize;
	lzma.next_out = bufferout;
	lzma.avail_out = BUFLEN;
	
	// Initialize a coder to the lzma_stream
	res = lzma_easy_encoder(&lzma, 6, LZMA_CHECK_CRC32);
	printf("lzma_easy_encoder()=%d\n", res);
	if (res!=LZMA_OK)
	{	printf("lzma_easy_encoder(6, LZMA_CHECK_CRC32)  failed with res=%d\n", res);
		lzma_end(&lzma);
		return 1;
	}
	
	res = lzma_code(&lzma, LZMA_RUN);
	printf("lzma_code(LZMA_RUN)=%d\n", res);
	printf("total_in=%d\n", lzma.total_in);
	printf("total_out=%d\n", lzma.total_out);
	if (res!=LZMA_OK)
	{	printf("lzma_code(LZMA_RUN)  failed with res=%d\n", res);
		lzma_end(&lzma);
		return 1;
	}
	
	res = lzma_code(&lzma, LZMA_FINISH);
	finalsize=lzma.total_out;
	printf("lzma_code(LZMA_FINISH)=%d and finalsize=%ld\n", res, finalsize);
	printf("total_in=%d\n", lzma.total_in);
	printf("total_out=%d\n", lzma.total_out);
	if (res!=LZMA_STREAM_END)
	{	printf("lzma_code(LZMA_FINISH) failed with res=%d\n", res);
		lzma_end(&lzma);
		return 1;
	}
	
	fout=fopen("/root/testlzma/file2.txt", "wb");
	if (!fout)
	{	printf ("Cannot open file2.txt\n");
		lzma_end(&lzma);
		return 1;
	}
	res=fwrite(bufferout, 1, finalsize, fout);
	printf ("fwrite(file2)=%d\n", res);
	fclose(fout);
	
	printf("----------------------------------------\n");

	// ======================================================================
	// ======================================================================
	
	// ---- 2. DECOMPRESSION
	fin=fopen("/root/testlzma/file2.txt", "rb");
	if (!fin)
	{	printf ("Cannot open file2.txt for read\n");
		lzma_end(&lzma);
		return 1;
	}
	origsize=fread(bufferin, 1, BUFLEN, fin);
	printf ("fread(file1)=%d\n", origsize);
	fclose(fin);
	
	// init lzma structures
	lzma.next_in = bufferin;
	lzma.avail_in = origsize;
	lzma.next_out = bufferout;
	lzma.avail_out = BUFLEN;
	
	// Initialize a coder to the lzma_stream
	res = lzma_auto_decoder(&lzma, 32*1024*1024, 0);
	printf("lzma_auto_decoder(&lzma, 32*1024*1024, 0)=%d\n", res);
	if (res!=LZMA_OK)
	{	printf("lzma_auto_decoder() failed with res=%d\n", res);
		lzma_end(&lzma);
		return 1;
	}
	
	res = lzma_code(&lzma, LZMA_RUN);
	finalsize=lzma.total_out;
	printf("lzma_code(LZMA_RUN)=%d\n", res);
	printf("total_in=%d\n", lzma.total_in);
	printf("total_out=%d\n", lzma.total_out);
	if (res!=LZMA_OK && res!=LZMA_STREAM_END)
	{	printf("lzma_code(LZMA_RUN) failed with res=%d\n", res);
		lzma_end(&lzma);
		return 1;
	}
	
	fout=fopen("/root/testlzma/file3.txt", "wb");
	if (!fout)
	{	printf ("Cannot open file3.txt\n");
		lzma_end(&lzma);
		return 1;
	}
	res=fwrite(bufferout, 1, finalsize, fout);
	printf ("fwrite(file3)=%d\n", res);
	fclose(fout);
	lzma_end(&lzma);

	return 0;
}
